"""Template loading utilities for emdash."""

from .loader import (
    load_template,
    load_template_for_agent,
    get_template_path,
    list_templates,
)

__all__ = ["load_template", "load_template_for_agent", "get_template_path", "list_templates"]
